/*
 * Copyright (c) 2015-2016, Technische Universität Dresden, Germany
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions
 *    and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors may be used to
 *    endorse or promote products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef INCLUDE_NITRO_LOG_MPI_RANK_ATTRIBUTE_HPP
#define INCLUDE_NITRO_LOG_MPI_RANK_ATTRIBUTE_HPP

#include <mpi.h>

namespace nitro
{
namespace log
{

    class mpi_rank_attribute
    {
        static int& get_mpi_rank()
        {
            static int mpi_rank_ = -1;

            return mpi_rank_;
        }

        static int& get_mpi_size()
        {
            static int mpi_size_ = -1;

            return mpi_size_;
        }

    public:
        static void initialize()
        {
            int init = 0;
            int fini = 0;

            MPI_Initialized(&init);
            MPI_Finalized(&fini);

            if (init && !fini)
            {
                MPI_Comm_rank(MPI_COMM_WORLD, &get_mpi_rank());
                MPI_Comm_size(MPI_COMM_WORLD, &get_mpi_size());
            }
        }

        mpi_rank_attribute()
        {
            if (get_mpi_rank() == -1)
            {
                initialize();
            }
        }

        int mpi_size() const
        {
            return get_mpi_size();
        }

        int mpi_rank() const
        {
            return get_mpi_rank();
        }
    };
}
} // namespace nitro::log

#endif // INCLUDE_NITRO_LOG_MPI_RANK_ATTRIBUTE_HPP
